/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <unistd.h>
#include <Ecore.h>
#include "thread.h"
#include "$(appName).h"

/**
 * @brief Function ran in a thread and responsible for time measurement.
 * The infinite loop implemented inside computes elapsed time and notifies the
 * main thread with current iteration number. Each loop iteration lasts 10 ms.
 * @param[in] data The data passed to the callback attachment function.
 * @param[in] thread The thread object.
 */
void thread_job(void *data, Ecore_Thread *thread)
{
	thread_info_s *th_info = NULL;
	int i = 0;

	if (!data || !thread) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid thread job input arguments.");
		return;
	}

	th_info = (thread_info_s *)data;

	for (i = 0; ; i++) {
		th_info->elapsedTime = ecore_time_get() - th_info->startTime;
		ecore_thread_feedback(thread, (void *)(uintptr_t)i);
		usleep(10000); /* One can have some real computation done here … */

		if (ecore_thread_check(thread))
			break;
	}
}

/**
 * @brief Function responsible for the main thread notification. Once the thread job
 * function calls ecore_thread_feedback, this function is invoked with time measurement
 * data passed in data argument. The passed time is converted to relevant data structure
 * and passed further to the main thread.
 * @param[in] data The data passed to the callback attachment function.
 * @param[in] thread The thread object.
 * @param[in] msg The thread message.
 */
void thread_job_notify(void *data, Ecore_Thread *thread, void *msg)
{
	thread_info_s *th_info = NULL;
	double time = 0;

	if (!data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid thread job notify input arguments.");
		return;
	}

	th_info = (thread_info_s *)data;
	time = th_info->elapsedTime * 100;

	th_info->lmsec = (long)(time) % 100;
	th_info->lsec = (long)(time / 100) % 60;
	th_info->lmin = (long)(time / 100 / 60) % 60;
	th_info->lhour = (long)(time / 100 / 60 / 60) % 60;

	tizen_error_e result = TIZEN_ERROR_UNKNOWN;

	/* stopwatch end */
	if (th_info->lhour == 59 && th_info->lmin == 59 && th_info->lsec == 59 && th_info->lmsec == 99) {
		if (th_info->time_stop_func)
			result = th_info->time_stop_func();

		if (result != TIZEN_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "Failed to stop the stopwatch: %d", result);
			return;
		}
	}

	if (th_info->time_update_func)
		th_info->time_update_func(th_info->lhour, th_info->lmin, th_info->lsec, th_info->lmsec);
}

/**
 * @brief Function responsible for the thread termination.
 * @param[in] data The data passed to the callback attachment function.
 * @param[in] thread The thread object.
 */
void thread_job_end(void *data, Ecore_Thread *thread)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "The stopwatch thread has stopped normally.");
}
